<?

namespace lib\usertype;

use Bitrix\Iblock;

/**
 * Реализация свойство «Расписание врача»
 * Class CUserTypeTimesheet
 * @package lib\usertype
 */

class CUserTypeTimesheet
{
    /**
     * Метод возвращает массив описания собственного типа свойств
     * @return array
     */
    public function GetUserTypeDescription()
    {
        return array(
            'USER_TYPE_ID' => 'user_timesheet', // уникальный идентификатор типа свойств
            'USER_TYPE' => 'TIMESHEET', // cимвольный код типа свойства
            'CLASS_NAME' => __CLASS__, // название класса который реализует логику типа свойства, чаще всего данный метод и логика свойства располагаются в одном классе, но можно разделить их на разные
            'DESCRIPTION' => 'Расписание специалиста', // название типа свойства которое будет отображаться в административной панели
            'PROPERTY_TYPE' => Iblock\PropertyTable::TYPE_STRING, // тип базового свойства, от которого наследуется стандартная логика работа со свойством
            'ConvertToDB' => [__CLASS__, 'ConvertToDB'], // конвертация данных перед сохранением в базу данных
            'ConvertFromDB' => [__CLASS__, 'ConvertFromDB'], // конвертация данных при извлечении из базы данных
            'GetPropertyFieldHtml' => [__CLASS__, 'GetPropertyFieldHtml'], // форма редактирования значения
        );
    }

    /**
     * Конвертация данных перед сохранением в базу данных, должен преобразовать значение свойства в формат пригодный для сохранения в базе данных и вернуть массив вида array("VALUE" => "...", "DESCRIPTION" => "..."), вызывается перед сохранением значения свойства в базе данных
     * @param $arProperty, метаданные свойства
     * @param $value, значение выбрнных свойств, массив вида array("VALUE" => значение, "DESCRIPTION" => описание)
     * @return mixed
     */
    public static function ConvertToDB($arProperty, $value)
    {
        // если $value не пустое и заполнено поле до которого часа
        if ($value['VALUE']['TIME_FROM'] != '' && $value['VALUE']['TIME_TO'] != '') {
            try {
                // кодируем данные которые будут записаны в базу данных
                $value['VALUE'] = base64_encode(serialize($value['VALUE']));
            } catch (Bitrix\Main\ObjectException $exception) {
                echo $exception->getMessage();
            }
        } else {
            $value['VALUE'] = '';
        }

        return $value;
    }

    /**
     * Конвертируем данные при извлечении из базы данных, должен преобразовать значение свойства из формата пригодного для сохранения в базе данных в формат обработки который увидим в админке, возвращает массив вида array("VALUE" => "...", "DESCRIPTION" => "...")
     * @param $arProperty, метаданные свойства
     * @param $value, значение свойств прочитанное из базы данных, массив вида array("VALUE" => значение, "DESCRIPTION" => описание)
     * @param string $format
     * @return mixed
     */
    public static function ConvertFromDB($arProperty, $value, $format = '')
    {
        // если $value не пустое
        if ($value['VALUE'] != '') {
            try {
                // раскодируем данные которые были записаны в базу данных
                $value['VALUE'] = base64_decode($value['VALUE']);
            } catch (Bitrix\Main\ObjectException $exception) {
                echo $exception->getMessage();
            }
        }

        return $value;
    }

    /**
     * Представление формы редактирования значения в админке, метод должен вернуть HTML отображения элемента управления для редактирования значений свойства в административной части
     * @param $arProperty, метаданные свойства
     * @param $value, значение свойства, массив вида array("VALUE" => значение, "DESCRIPTION" => описание)
     * @param $arHtmlControl, имена элементов управления для заполнения значения свойства и его описания, массив вида array("VALUE" => html безопасное имя для значения, "DESCRIPTION" => html безопасное имя для описания, "MODE" => может принимать зачение "FORM_FILL" при вызове из формы редактирования элемента или "iblock_element_admin" при редактировании в режиме просмотра списка элементов, а также "EDIT_FORM" при редактировании инфоблока, "FORM_NAME" => имя формы в которую будет встроен элемент управления)
     */
    public static function GetPropertyFieldHtml($arProperty, $value, $arHtmlControl)
    {
        $weekDays = [
            'mon' => 'Понедельник',
            'tue' => 'Вторник',
            'wed' => 'Среда',
            'thu' => 'Четверг',
            'fri' => 'Пятница',
            'sat' => 'Суббота',
            'sun' => 'Воскресенье',
        ];

        // ID для js
        $itemId = 'row_' . substr(md5($arHtmlControl['VALUE']), 0, 10);
        // устанавливаем правильную кодировку
        $fieldName =  htmlspecialcharsbx($arHtmlControl['VALUE']);
        // unserialize преобразование серилизованных массивов, htmlspecialcharsback нужен для того, чтобы избавиться от многобайтовых символов из-за которых не работает unserialize
        $arValue = unserialize(htmlspecialcharsback($value['VALUE']), [stdClass::class]);

        // формируем раскрывающийся список select с днями недели
        $select = '<select class="week_day" name="' . $fieldName . '[WEEK_DAY]">';
        foreach ($weekDays as $key => $day) {
            if ($arValue['WEEK_DAY'] == $key) {
                $select .= '<option value="' . $key . '" selected="selected">' . $day . '</option>';
            } else {
                $select .= '<option value="' . $key . '">' . $day . '</option>';
            }
        }
        $select .= '</select>';

        // HTML каркас для вывода с ID для js
        $html = '<div class="property_row" id="' . $itemId . '">';
        // HTML каркас для вывода
        $html .= '<div class="reception_time">';
        // вставляем в каркас раскрывающийся список
        $html .= $select;
        // вставляем пустое значение для время приема с
        $timeFrom = ($arValue['TIME_FROM']) ? $arValue['TIME_FROM'] : '';
        // вставляем пустое значение для время приема до
        $timeTo = ($arValue['TIME_TO']) ? $arValue['TIME_TO'] : '';
        // формируем input поле для время приема с
        $html .= '&nbsp;время приёма: с&nbsp;<input type="time" name="' . $fieldName . '[TIME_FROM]" value="' . $timeFrom . '">';
        // формируем input поле для время приема до
        $html .= '&nbsp;по&nbsp;<input type="time" name="' . $fieldName . '[TIME_TO]" value="' . $timeTo . '">';
        // кнопка удаления input
        if ($timeFrom != '' && $timeTo != '') {
            $html .= '&nbsp;&nbsp;<input type="button" style="height: auto;" value="x" title="Удалить" onclick="document.getElementById(\'' . $itemId . '\').parentNode.parentNode.remove()" />';
        }
        $html .= '</div>';
        $html .= '</div><br/>';

        return $html;
    }
}
